// Demo 18: SearchApplet.java

//   A Linear Search and Table Lookup Demo
//                 A Java Applet
//
//   Lookup a name, find the corresponding
//            Social Security Number
//
//            by Charles W. Neville, 1998
//         Copyright Charles W. Neville, 1998

// This Applet illustrates an important application of LINER SEARCH, namely
// TABLE LOOKUP.  Table lookup is easy to understand; it works just as you
// do when you look something up in a table.  For example, pretend you had the
// following table of names and social security numbers:
//
//  colA       colB
//  Name       SSN
//  BETTY      123456789
//  ARCHIE     999999999
//  JOE        043047654
//  NELI       042838353
//  ARTHUR     555555555
//
// If you wanted to look up, say, NELI's Social Security number, you would just 
// run your finger down the Name column, column A, until you found NELI, and 
// then across to the SSN column, column B, to get her Social Security number, 
// 042838353.  The TWO ARRAY METHOD of TABLE LOOKUP works the same way.  You 
// implement a table with pair of arrays, which we shall call colA and colB.
// In the case just described, the colA array would hold names, and the colB
// array would hold the corresponding Social Security numbers.  To find NELI's 
// Social Security number, you would use a LINEAR SEARCH method to find the 
// ARRAY INDEX of the array box where NELI's name first appears in colA, and 
// then you would print out the Social Security number contained in the 
// corresponding array box of colB.
//
// Of course, the TWO ARRAY METHOD of TABLE LOOKUP depends on a LINEAR SEARCH 
// method.  The linear search method we shall use works just as you do when you 
// run your finger down the Name column of the table until you find what you 
// are looking for.  The only difficulty arises when the name the program is 
// looking for does not appear in the colA array.  Humans are smart enough to 
// stop at the end of the table, but computer programs are not smart enough to 
// stop unless we put in special code to make them stop.  The special code we 
// shall use is called the sentinel technique, and it is similar to reading-
// and-processing-up-to-a-stopping-signal.  What we shall do is put the name we 
// are looking for in the last box of the colA array; this way, the linear 
// search method is guaranteed to find it and stop.  Then we shall write code 
// to see if the linear search method found the name before it got to the last 
// box of array colA, in which case it really found the name, or if it found 
// the name in the last array box, in which case it found our "sentinel" 
// instead of the actual name.
//
// Try compiling and running this applet with the following names as data to be 
// be entered: NELI, Archie, ARCHIE, BETTY, and VERONICA.  You should get
//
// NELI has SS Num 42838353
// Archie not found
// ARCHIE has SS Num 999999999
// BETTY has SS Num 123456789
// VERONICA not found

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

public class SearchApplet extends Applet implements ActionListener {

   // Table column arrays
   String[] colA = {
     "BETTY",
     "ARCHIE",
     "JOE",
     "NELI",
     "ARTHUR",
     "NO NAME TO BE REPLACED by x"
   };

   int[] colB = {
      123456789,
      999999999, 
       43047654, // one of Java's quirks is that numbers in code
       42838353, // beginning with 0 are interpreted in octal
      555555555
   };

   // found state variable -- did we find what we are looking for?
   boolean found;

   // User interface variables
   Label inputPrompt;     // prompt on Applet Panel for TextField inputField
   TextField inputField;  // where we enter name to be searched for. 
   Label outputPrompt;    // prompt on Applet Panel for TextField outputField
   TextField outputField; // where the corresponding Social Security Number
                          // appears
   Button searchButton;   // press to begin search

   // build Applet panel and initialize, as usual.
   public void init() {

      // create new instances of labels, textfields and buttons
      inputPrompt = new Label("Enter name: ");
      inputField = new TextField(30);  // 30 characters possible input
      outputPrompt = new Label("SSN: ");  //
      outputField = new TextField(30);  // 30 characters possible output
      searchButton = new Button("Search");

     // add components to Applet Panel
     add(inputPrompt);
     add(inputField);
     add(outputPrompt);
     add(outputField);
     add(searchButton);

     // Register searchButton as the ActionListener
     // (ActionEvent processors) for this Applet
     searchButton.addActionListener(this);

   }  // init

   //  Linear Search Method
   //
   //  uses sentinel technique
   //
   //  parameters:
   //    A : array to be searched
   //    x : thing to search for
   //
   //  returns: position of x in A, if found
   //           A.length - 1 if not found
   //
   //  special considerations:
   //     A must have an extra box for sentinel value
   //
   //  side effects:
   //     x is written to last box of array A
   //   
   public int linSearch(String[] A, String x) {

      int i;

      A[A.length - 1] = x;  // insert x as sentinel value in last box
      i = 0;
      // careful!  Compare strings using .equals() instead of =
      // !x.equals(A[i]) means x!= A[i] in the sense that
      // the content of x != the content of A[i]
      while ( !x.equals(A[i]) ) {
         i++;
      }
      return i;
   } // linSearch

   // Table LookUp Method
   //
   // parameters:
   //   A : table column A Array
   //   B : table column B Array
   //   x : thing to look up in column A
   //
   //   returns: corresponding column B value if found
   //            -1 (an impossible Social Security number) if not found
   //   sets: found object state variable -- did we find x or not?
   //
   // special considerations:                        
   //   array A must have an extra box for sentinel value
   //
   //  side effects:
   //   x is written to last box of colA array
   //
   public int tableLookUp(String x, String[] A, int[] B) {

      int i;

      i = linSearch(A, x);
      if(i < A.length - 1) {
         // we found it, so
         found = true;
         return B[i];
      }
      else {
         // we didn't find it, so
         found = false;
         return -1;
      }
   }  // tableLookUp

   // process button click -- look up name 
   public void actionPerformed(ActionEvent e) {

      String name;  // name we are looking for
      int SSN;      // corresponding Social Security Number

      if( e.getActionCommand().equals("Search") ) {

        // we pressed Search button, so
        name = inputField.getText();
        SSN = tableLookUp(name, colA, colB);
        if(found) {
          outputField.setText(name + " has SS Num " + SSN);
        }
        else {
          outputField.setText(name + " not found");
        }
      }

      repaint();

   } // actionPerformed

}  // class SearchApplet